@@ -219,6 +219,9 @@ typedef struct scap_threadinfo
 	char root[SCAP_MAX_PATH_SIZE];
 	int filtered_out; ///< nonzero if this entry should not be saved to file
 	scap_fdinfo* fdlist; ///< The fd table for this process
+	uint64_t clone_ts;
+	int32_t tty;
+	
 	UT_hash_handle hh; ///< makes this structure hashable
 }scap_threadinfo;
 
@@ -231,8 +234,15 @@ typedef void (*proc_entry_callback)(void* context,
 /*!
   \brief Arguments for scap_open
 */
+typedef enum {
+	SCAP_MODE_CAPTURE,
+	SCAP_MODE_LIVE,
+	SCAP_MODE_NODRIVER
+} scap_mode_t;
+
 typedef struct scap_open_args
 {
+	scap_mode_t mode;
 	const char* fname; ///< The name of the file to open. NULL for live captures.
 	proc_entry_callback proc_callback; ///< Callback to be invoked for each thread/fd that is extracted from /proc, or NULL if no callback is needed.
 	void* proc_callback_context; ///< Opaque pointer that will be included in the calls to proc_callback. Ignored if proc_callback is NULL.
@@ -869,12 +879,17 @@ int32_t scap_enable_dynamic_snaplen(scap_t* handle);
 int32_t scap_disable_dynamic_snaplen(scap_t* handle);
 void scap_proc_free_table(scap_t* handle);
 void scap_refresh_iflist(scap_t* handle);
+void scap_refresh_proc_table(scap_t* handle);
 void scap_set_refresh_proc_table_when_saving(scap_t* handle, bool refresh);
 uint64_t scap_ftell(scap_t *handle);
 void scap_fseek(scap_t *handle, uint64_t off);
 int32_t scap_enable_tracers_capture(scap_t* handle);
 uint64_t scap_get_unexpected_block_readsize(scap_t* handle);
 int32_t scap_proc_add(scap_t* handle, uint64_t tid, scap_threadinfo* tinfo);
+int32_t scap_fd_add(scap_threadinfo* tinfo, uint64_t fd, scap_fdinfo* fdinfo);
+scap_dumper_t *scap_memory_dump_open(scap_t *handle, uint8_t* targetbuf, uint64_t targetbufsize);
+int32_t compr(uint8_t* dest, uint64_t* destlen, const uint8_t* source, uint64_t sourcelen, int level);
+uint8_t* scap_get_memorydumper_curpos(scap_dumper_t *d);
 
 #ifdef __cplusplus
 }
