@@ -340,35 +340,36 @@ public:
 		TYPE_NCHILDS = 9,
 		TYPE_PPID = 10,
 		TYPE_PNAME = 11,
-		TYPE_APID = 12,
-		TYPE_ANAME = 13,
-		TYPE_LOGINSHELLID = 14,
-		TYPE_DURATION = 15,
-		TYPE_FDOPENCOUNT = 16,
-		TYPE_FDLIMIT = 17,
-		TYPE_FDUSAGE = 18,
-		TYPE_VMSIZE = 19,
-		TYPE_VMRSS = 20,
-		TYPE_VMSWAP = 21,
-		TYPE_PFMAJOR = 22,
-		TYPE_PFMINOR = 23,
-		TYPE_TID = 24,
-		TYPE_ISMAINTHREAD = 25,
-		TYPE_EXECTIME = 26,
-		TYPE_TOTEXECTIME = 27,
-		TYPE_CGROUPS = 28,
-		TYPE_CGROUP = 29,
-		TYPE_VTID = 30,
-		TYPE_VPID = 31,
-		TYPE_THREAD_CPU = 32,
-		TYPE_THREAD_CPU_USER = 33,
-		TYPE_THREAD_CPU_SYSTEM = 34,
-		TYPE_THREAD_VMSIZE = 35,
-		TYPE_THREAD_VMRSS = 36,
-		TYPE_THREAD_VMSIZE_B = 37,
-		TYPE_THREAD_VMRSS_B = 38,
-		TYPE_SID = 39,
-		TYPE_SNAME = 40,
+		TYPE_PCMDLINE = 12,
+		TYPE_APID = 13,
+		TYPE_ANAME = 14,
+		TYPE_LOGINSHELLID = 15,
+		TYPE_DURATION = 16,
+		TYPE_FDOPENCOUNT = 17,
+		TYPE_FDLIMIT = 18,
+		TYPE_FDUSAGE = 19,
+		TYPE_VMSIZE = 20,
+		TYPE_VMRSS = 21,
+		TYPE_VMSWAP = 22,
+		TYPE_PFMAJOR = 23,
+		TYPE_PFMINOR = 24,
+		TYPE_TID = 25,
+		TYPE_ISMAINTHREAD = 26,
+		TYPE_EXECTIME = 27,
+		TYPE_TOTEXECTIME = 28,
+		TYPE_CGROUPS = 29,
+		TYPE_CGROUP = 30,
+		TYPE_VTID = 31,
+		TYPE_VPID = 32,
+		TYPE_THREAD_CPU = 33,
+		TYPE_THREAD_CPU_USER = 34,
+		TYPE_THREAD_CPU_SYSTEM = 35,
+		TYPE_THREAD_VMSIZE = 36,
+		TYPE_THREAD_VMRSS = 37,
+		TYPE_THREAD_VMSIZE_B = 38,
+		TYPE_THREAD_VMRSS_B = 39,
+		TYPE_SID = 40,
+		TYPE_SNAME = 41,
 	};
 
 	sinsp_filter_check_thread();
@@ -592,8 +593,8 @@ public:
 private:
 	int32_t extract_arg(string fldname, string val, OUT const struct ppm_param_info** parinfo);
 	inline int64_t* extract_duration(uint16_t etype, sinsp_tracerparser* eparser);
-	uint8_t* extract_args(sinsp_partial_tracer* pae);
-	uint8_t* extract_arg(sinsp_partial_tracer* pae);
+	uint8_t* extract_args(sinsp_partial_tracer* pae, OUT uint32_t *len);
+	uint8_t* extract_arg(sinsp_partial_tracer* pae, OUT uint32_t *len);
 
 	int32_t m_argid;
 	string m_argname;
@@ -669,7 +670,7 @@ public:
 
 private:
 	int32_t extract_arg(string fldname, string val);
-	inline uint8_t* extract_tracer(sinsp_evt *evt, sinsp_partial_tracer* pae);
+	inline uint8_t* extract_tracer(sinsp_evt *evt, sinsp_partial_tracer* pae, OUT uint32_t* len);
 	inline bool compare_tracer(sinsp_evt *evt, sinsp_partial_tracer* pae);
 
 	bool m_is_compare;
